package cn.com.qmth.examcloud.core.basic.api.controller;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.StringEscapeUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.ErrorMsg;
import cn.com.qmth.examcloud.commons.web.helpers.page.PageInfo;
import cn.com.qmth.examcloud.commons.web.security.annotation.Uac;
import cn.com.qmth.examcloud.commons.web.security.entity.AccessUser;
import cn.com.qmth.examcloud.commons.web.security.enums.RoleMeta;
import cn.com.qmth.examcloud.commons.web.security.enums.UacPolicy;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.controller.bean.FullUserInfo;
import cn.com.qmth.examcloud.core.basic.api.controller.bean.UserForm;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.RoleRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRoleRelationRepo;
import cn.com.qmth.examcloud.core.basic.dao.constants.Consts;
import cn.com.qmth.examcloud.core.basic.dao.entity.Org;
import cn.com.qmth.examcloud.core.basic.dao.entity.RoleEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserRoleRelationEntity;
import cn.com.qmth.examcloud.core.basic.service.bean.UserInfo;
import cn.com.qmth.examcloud.core.basic.service.impl.UserServiceImpl;
import io.swagger.annotations.ApiOperation;

/**
 * 用户服务API Created by songyue on 17/1/13.
 */
@Transactional
@RestController
@RequestMapping("${$rmp.ctr.basic}/user")
public class UserController extends ControllerSupport {

	@Autowired
	UserServiceImpl userService;

	@Autowired
	UserRepo userRepo;

	@Autowired
	OrgRepo orgRepo;

	@Autowired
	RoleRepo roleRepo;

	@Autowired
	UserRoleRelationRepo userRoleRelationRepo;

	/**
	 * 重新定义
	 *
	 * @author WANGWEI
	 * @param curPage
	 * @param pageSize
	 * @param rootOrgId
	 * @param loginName
	 * @param name
	 * @param enable
	 * @param roleId
	 * @param type
	 * @return
	 */
	@ApiOperation(value = "查询所有用户", notes = "")
	@GetMapping("/all/{curPage}/{pageSize}")
	public PageInfo<FullUserInfo> getAllUser(@PathVariable Integer curPage,
			@PathVariable Integer pageSize, @RequestParam Long rootOrgId,
			@RequestParam String loginName, @RequestParam String name, @RequestParam String enable,
			@RequestParam(required = false) Long roleId, @RequestParam String type) {

		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if ((!isSuperAdmin()) && (!rootOrgId.equals(accessUser.getRootOrgId()))) {
			throw new StatusException("B-150001", "非法请求");
		}

		Org rootOrg = orgRepo.findOne(rootOrgId);
		if (null == rootOrg) {
			throw new StatusException("B-150003", "机构不存在");
		}
		if (null != rootOrg.getParentId()) {
			throw new StatusException("B-150004", "机构错误");
		}

		String roleCode = null;
		if (null != roleId) {
			RoleEntity roleEntity = roleRepo.findOne(roleId);
			if (null == roleEntity) {
				throw new StatusException("B-150002", "角色不存在");
			}
			roleCode = roleEntity.getCode();
		}

		Pageable pageable = new PageRequest(curPage - 1, pageSize, Sort.Direction.DESC,
				"update_time");

		loginName = toSqlSearchPattern(loginName);
		name = toSqlSearchPattern(name);

		Page<UserEntity> userList = null;
		if (null == roleCode) {
			userList = userRepo.findAll(rootOrgId, name, loginName, type, pageable);
		} else {
			userList = userRepo.findAll(rootOrgId, name, loginName, type, roleCode, pageable);
		}

		Iterator<UserEntity> iterator = userList.iterator();

		List<FullUserInfo> fullUserInfoList = Lists.newArrayList();
		while (iterator.hasNext()) {
			UserEntity next = iterator.next();
			FullUserInfo bean = new FullUserInfo();
			bean.setId(next.getId());
			bean.setLoginName(next.getLoginName());
			bean.setName(next.getName());
			bean.setRootOrgId(next.getRootOrgId());
			bean.setUpdateTime(next.getUpdateTime());
			bean.setCreationTime(next.getCreationTime());
			bean.setOrgId(next.getOrgId());
			if (null != bean.getOrgId()) {
				Org org = orgRepo.findOne(Long.valueOf(bean.getOrgId()));
				if (null != org) {
					bean.setOrgName(org.getName());
				}
			}
			bean.setRootOrgName(rootOrg.getName());
			bean.setEnable(next.getEnable());

			List<UserRoleRelationEntity> relationList = userRoleRelationRepo
					.findAllByUserId(next.getId());
			List<String> roleNameList = Lists.newArrayList();
			List<Long> roleIdList = Lists.newArrayList();
			List<String> roleCodeList = Lists.newArrayList();
			for (UserRoleRelationEntity cur : relationList) {
				RoleEntity curRoleEntity = roleRepo.findByCode(cur.getRoleCode());
				if (null == curRoleEntity) {
					throw new StatusException("B-150002", "角色错误");
				}
				roleNameList.add(curRoleEntity.getName());
				roleIdList.add(curRoleEntity.getId());
				roleCodeList.add(curRoleEntity.getCode());
			}
			bean.setRoleNamesStr(StringUtils.join(roleNameList, ","));
			bean.setRoleIds(roleIdList);
			bean.setRoleCodes(roleCodeList);

			fullUserInfoList.add(bean);
		}

		PageInfo<FullUserInfo> ret = new PageInfo<FullUserInfo>();
		ret.setList(fullUserInfoList);
		ret.setTotal(userList.getTotalElements());
		return ret;
	}

	@ApiOperation(value = "按id查询用户", notes = "id查询")
	@GetMapping("/{id}")
	public ResponseEntity getUserById(@PathVariable long id) {
		return new ResponseEntity(userRepo.findOne(id), HttpStatus.OK);
	}

	@ApiOperation(value = "按orgId查询用户", notes = "机构id查询机构用户")
	@GetMapping("/org/{orgId}")
	public List<UserEntity> getUserByOrgId(@PathVariable long orgId) {
		List<UserEntity> userList = userRepo.findByOrgId(orgId);
		return userList;
	}

	@ApiOperation(value = "按rootOrgId查询用户", notes = "根机构id查询机构用户")
	@GetMapping("/rootOrg/{rootOrgId}")
	public List<UserEntity> getUserByRootOrgId(@PathVariable long rootOrgId) {
		List<UserEntity> userList = userRepo.findByRootOrgId(rootOrgId);
		return userList;
	}

	/**
	 * 重构 2018年6月26日
	 *
	 * @author WANGWEI
	 * @param userForm
	 * @return
	 */
	@ApiOperation(value = "新增用户", notes = "新增")
	@PostMapping
	public Map<String, Object> addUser(@RequestBody UserForm userForm) {
		userForm.setId(null);
		return insertOrUpdateUser(userForm);
	}

	/**
	 * 重构 2018年6月26日
	 *
	 * @author WANGWEI
	 * @param userForm
	 * @return
	 */
	@ApiOperation(value = "更新用户", notes = "更新")
	@PutMapping
	public Map<String, Object> updateUser(@RequestBody UserForm userForm) {
		if (null == userForm.getId()) {
			throw new StatusException("B-150009", "user ID is null");
		}

		UserEntity userEntity = userRepo.findOne(userForm.getId());
		if (null == userEntity) {
			throw new StatusException("B-150010", "用户不存在");
		}

		if (!userEntity.getRootOrgId().equals(userForm.getRootOrgId())) {
			throw new StatusException("B-150010", "顶级机构错误");
		}

		userForm.setPassword(userEntity.getPassword());

		return insertOrUpdateUser(userForm);
	}

	/**
	 * 新增或更新用户
	 *
	 * @author WANGWEI
	 * @param userForm
	 * @return
	 */
	private Map<String, Object> insertOrUpdateUser(UserForm userForm) {
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		Long rootOrgId = userForm.getRootOrgId();
		Org rootOrg = orgRepo.findOne(rootOrgId);
		if (null == rootOrg) {
			throw new StatusException("B-150003", "机构不存在");
		}
		if (null != rootOrg.getParentId()) {
			throw new StatusException("B-150004", "机构错误");
		}

		if ((!isSuperAdmin()) && (!rootOrgId.equals(accessUser.getRootOrgId()))) {
			throw new StatusException("B-150005", "无权操作");
		}

		UserEntity userEntity = new UserEntity();
		userEntity.setId(userForm.getId());
		userEntity.setEnable(userForm.getEnable());
		userEntity.setLoginName(userForm.getLoginName());
		userEntity.setMobile(userForm.getMobile());
		userEntity.setName(userForm.getName());
		userEntity.setOrgId(userForm.getOrgId());
		userEntity.setRootOrgId(rootOrgId);
		userEntity.setPassword(userForm.getPassword());
		userEntity.setUpdateTime(new Date());

		UserEntity saved = userService.save(userEntity);

		List<UserRoleRelationEntity> userRoles = Lists.newArrayList();
		List<Long> roleIds = userForm.getRoleIds();
		for (Long cur : roleIds) {
			RoleEntity curRoleEntity = roleRepo.findOne(cur);
			if (null == curRoleEntity) {
				throw new StatusException("B-150006", "角色错误");
			}
			if (curRoleEntity.getCode().equals(RoleMeta.SUPER_ADMIN.getCode())) {
				throw new StatusException("B-150007", "不允许新增或修改超级管理员");
			}
			UserRoleRelationEntity relation = new UserRoleRelationEntity(saved.getId(),
					curRoleEntity.getId(), curRoleEntity.getCode());
			userRoles.add(relation);
		}

		userRoleRelationRepo.deleteByUserId(saved.getId());
		List<UserRoleRelationEntity> savedRelationList = userRoleRelationRepo.save(userRoles);

		Map<String, Object> ret = Maps.newHashMap();
		ret.put("userId", saved.getId());
		ret.put("loginName", saved.getLoginName());
		ret.put("roleList", savedRelationList);
		return ret;
	}

	@ApiOperation(value = "重置用户密码", notes = "重置密码")
	@PutMapping("/resetPass/{id}")
	public void resetPass(@PathVariable String id) {
		List<Long> ids = Stream.of(id.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long userId : ids) {
			UserEntity user = userRepo.findOne(userId);
			user.setPassword(Consts.DEFAULT_PASSWORD);
			userRepo.save(user);
		}
	}

	@ApiOperation(value = "启用用户", notes = "启用用户")
	@PutMapping("/enable/{ids}")
	public List<String> enableUser(@PathVariable String ids) {
		List<Long> userIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		List<String> ret = Lists.newArrayList();
		for (Long userId : userIds) {
			UserEntity user = userRepo.findOne(userId);
			user.setEnable(true);
			userRepo.save(user);
			ret.add(user.getId() + ":" + user.getName());
		}
		return ret;
	}

	@ApiOperation(value = "禁用用户", notes = "禁用用户")
	@PutMapping("/disable/{ids}")
	public List<String> disableUser(@PathVariable String ids) {
		List<Long> userIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		List<String> ret = Lists.newArrayList();
		for (Long userId : userIds) {
			UserEntity user = userRepo.findOne(userId);
			user.setEnable(false);
			userRepo.save(user);
			ret.add(user.getId() + ":" + user.getName());
		}
		return ret;
	}

	@ApiOperation(value = "修改用户密码", notes = "修改密码")
	@PutMapping("/password")
	public ResponseEntity updatePass(@RequestParam long userId, @RequestParam String password) {
		String realPassword = StringEscapeUtils.unescapeJavaScript(password);
		userRepo.updatePasswordById(userId, realPassword);
		return new ResponseEntity(HttpStatus.OK);
	}

	@ApiOperation(value = "按id删除用户", notes = "删除")
	@DeleteMapping("/{ids}")
	@Uac(roles = {RoleMeta.SUPER_ADMIN}, policy = UacPolicy.IN)
	public ResponseEntity deleteUser(@PathVariable String ids) {
		List<Long> userIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long userId : userIds) {
			userRepo.delete(userId);
		}
		return new ResponseEntity(HttpStatus.OK);
	}

	@ApiOperation(value = "一般登录", notes = "登录")
	@PostMapping("/login")
	@Deprecated
	public ResponseEntity login(@RequestParam String loginName, @RequestParam String password) {
		try {
			UserInfo userInfo = userService.login(loginName, password);
			return new ResponseEntity(userInfo, HttpStatus.OK);
		} catch (Exception e) {
			return new ResponseEntity(new ErrorMsg(e.getMessage()),
					HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}

	@ApiOperation(value = "二级登录", notes = "二级登录")
	@PostMapping("/login/{orgId}")
	@Deprecated
	public ResponseEntity login(@PathVariable long orgId, @RequestParam String loginName,
			@RequestParam String password) {
		try {
			UserInfo userInfo = userService.login(orgId, loginName, password);
			return new ResponseEntity(userInfo, HttpStatus.OK);
		} catch (Exception e) {
			return new ResponseEntity(new ErrorMsg(e.getMessage()),
					HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}

	@ApiOperation(value = "第三方机构接入", notes = "第三方机构接入")
	@PostMapping("/thirdPartyAccess/{orgId}")
	@Deprecated
	public ResponseEntity thirdPartyAccess(@PathVariable long orgId, @RequestParam String userid,
			@RequestParam String appid, @RequestParam String timestamp,
			@RequestParam String token) {
		try {
			UserInfo userInfo = userService.thirdPartyAccess(orgId, userid, appid, timestamp,
					token);
			return new ResponseEntity(userInfo, HttpStatus.OK);
		} catch (Exception e) {
			return new ResponseEntity(new ErrorMsg(e.getMessage()),
					HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}

	@ApiOperation(value = "登出", notes = "登出")
	@PostMapping("/logout")
	@Deprecated
	public ResponseEntity logout(HttpServletRequest request) {
		AccessUser accessUser = null;
		userService.logout(accessUser);
		return new ResponseEntity(HttpStatus.OK);
	}

	@ApiOperation(value = "查询评卷员", notes = "查询")
	@GetMapping("/marker")
	public ResponseEntity getMarker(HttpServletRequest request) {
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		return new ResponseEntity(userService.getMarker(accessUser.getRootOrgId()), HttpStatus.OK);
	}

	@ApiOperation(value = "查询评卷员带分页", notes = "查询评卷员带分页")
	@GetMapping("/all/marker/{curPage}/{pageSize}")
	public ResponseEntity getAllMark(@ModelAttribute UserEntity user, @PathVariable Integer curPage,
			@PathVariable Integer pageSize, HttpServletRequest request) {
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if (accessUser != null) {
			user.setRootOrgId(accessUser.getRootOrgId());
			return new ResponseEntity(
					userService.getAllMaker(user, new PageRequest(curPage - 1, pageSize)),
					HttpStatus.OK);
		} else {
			return new ResponseEntity(new PageImpl<UserEntity>(new ArrayList<UserEntity>()),
					HttpStatus.OK);
		}
	}

}
